require 'vikiwikiplugin'

module VikiWiki
  module Plugins
    class Tidy
      include BaseModule
      class Error < StandardError; end

      TIDY_PATH = "/usr/bin/tidy"
      #TIDY_PATH = "/home/itc/nakatuka/bin/tidy"
      DEFAULT_TIDYOPT  = "-wrap 80"
      ENCODING  = {"e" => "-raw", "s" => "-shiftjis", "u" => "-utf8"}

      if !defined?(LBL) then
        LBL = {
          "STATICLIST" => "Static HTML Page List",
          "RESULT"     => "Check Result",
          "SELECT"     => "Select",
          "TIDYED"     => "Checked?",
          "NAME"       => "Page Name",
          "MODTIME"    => "Modified Time",
          "CHECKTIDY"  => "Check by Tidy"
        }
      end
      if !defined?(MSG) then
        MSG = {
          "ADMINONLY"  => "This page can be used only by the administrator."
        }
      end

      private

      def tidyed?(path)
        tidy_pat = %r|<meta name="generator" content="HTML Tidy for.*/>|
        return (tidy_pat =~ File.read(path).gsub(/\n/, ""))
      end

      def checked?(name)
        return (@sys.cgi.params["tidy_#{name}"][0] == "on")
      end

      def modtime(path)
        return File.stat(path).mtime.strftime("%Y-%m-%d %H:%M:%S")
      end

      def each_static_html
        @sys.pages.to_a.each do |page|
          name = page.name; aliasname = page.alias
          htmlpath = check_path("#{@sys['STATICDIR']}/#{name}.html")
          htmluri = check_path("#{@sys['STATICURI']}/#{name}.html")
          if FileTest.exist?(htmlpath) then
            yield(name, aliasname, htmlpath, htmluri)
          end
        end
      end

      def check_path(path)
        if /[|&;`$<>]/ =~ path then
          raise Error, "check_path insecure error."
        end
        path.untaint; return path
      end

      public

      def ondesc; (<<-EODESC); end
This plugin check the static pages in {{VikiWiki}} by [http://www.w3.org/People/Raggett/tidy/ Tidy]. a parameter of the plugin is tidy's option for checking the static pages.
{{{
#tidy
#tidy [tidy-option]
}}}
      EODESC

      def safe; 1; end

      def onview
        if @sys.user != @sys['ADMIN'] then
          return (<<-EOHTML)
<div class="error"><p>
<em>#{MSG["ADMINONLY"]}</em>
</p></div>
          EOHTML
        end

        list = "<h2>#{LBL["STATICLIST"]}</h2>\n<table>\n"
        list << (<<-EOHTML)
<tr>
<td>#{LBL["SELECT"]}</td><td>#{LBL["TIDYED"]}</td><td>#{LBL["NAME"]}</td><td>#{LBL["MODTIME"]}</td>
</tr>
        EOHTML

        each_static_html do |name, aliasname, htmlpath, htmluri|
          tidyed = tidyed?(htmlpath)
          list << (<<-EOHTML)
<tr>
<td>#{checkbox("tidy_#{name}", "on", !tidyed)}</td>
<td>#{tidyed ? "yes" : "no"}</td>
<td><a href="#{htmluri}">#{aliasname}</a></td><td>#{modtime(htmlpath)}</td>
</tr>
          EOHTML
        end

        list << "</table>\n"
        return plugin_form { (<<-EOHTML) }
#{@result}
#{list}
#{submit("submit", LBL["CHECKTIDY"])}
        EOHTML
      end

      def onpost
        tidyopt = (@prms[0] || DEFAULT_TIDYOPT).gsub(/\"(.*)\"/, '\1')
        tidyopt = check_path("-m #{ENCODING[@sys.code]} #{tidyopt}")
        tidycmd = check_path("#{TIDY_PATH}")

        @result = "<h2>#{LBL["RESULT"]}</h2>\n<ul>\n"

        each_static_html do |name, aliasname, htmlpath, htmluri|
          if checked?(name) then
            mesg = CGI::escapeHTML(`#{tidycmd} #{tidyopt} #{htmlpath} 2>&1`)
            mesg.gsub!(/\n\n\n#{(<<EOPAT)}/, "")
To learn more about HTML Tidy see http://tidy.sourceforge.net
Please send bug reports to html-tidy@w3.org
HTML and CSS specifications are available from http://www.w3.org/
Lobby your company to join W3C, see http://www.w3.org/Consortium
EOPAT
            mesg.chomp!
            ($? >> 8).zero? || (mesg = "<b>#{mesg}</b>")

            @result << (<<-EOHTML)
<li><a href="#{htmluri}">#{aliasname}</a> -- #{modtime(htmlpath)}<br />
<pre>
#{mesg}
</pre>
</li>
            EOHTML
          end
        end
        @result << "</ul>\n<hr />\n"
      end
    end
  end
end
